package sprite;
import java.awt.*;

/**
 * A Bullet is a special type of Sprite.  A SpritePanel keeps track of bullets separately
 * from other sprites .  The default behavior is that when a sprite contains the location 
 * of a bullet, both the sprite and the bullet die, and if the bullet's "exploding" property
 * is true then an Explosion sprite is placed at the location of the bullet.  It is possible
 * to make sprites immune to bullets.  By default, a bullet will also die when it moves outside
 * the SpritePanel.  (One way to add a bullet to a panel is by firing a Gun sprite.)  Bullets
 * are shown by default as small black ovals. You shouldn't make them too big: only the center
 * of the bullet is used when testing if it has hit a sprite.
 */
public class Bullet extends Sprite {
	
	private Color color;
	private boolean exploding;
	
	/**
	 * Creates a bullet at a specified location and with a specified velocity.
	 * @param x the x-component of the initial location
	 * @param y the y-component of the initial location
	 * @param dx the x-component of the velocity
	 * @param dy the y-component of the velocity
	 */
	public Bullet(double x, double y, double dx, double dy) {
		setLocation(x,y);
		setVelocity(dx,dy);
		setExitPolicy(EXIT_POLICY_DIE);
		color = Color.BLACK;
		setSpriteWidth(5);
		setSpriteHeight(5);
		exploding = true;
	}
	
	/**
	 * Change the color of this Bullet.  If the color is null, then black is used.
	 */
	public void setColor(Color c) {
		color = c;
		if (color == null) {
			color = Color.BLACK;
		}
	}
	
	public void draw(Graphics g) {
		int centerX = (int)getX();
		int centerY = (int)getY();
		g.setColor(color);
		g.fillOval( centerX - getSpriteWidth()/2, centerY - getSpriteHeight()/2, getSpriteWidth(), getSpriteHeight() );
	}
	
	/**
	 * Sets the "exploding" property of this Bullet to true or false.  When this property is true,
	 * then when the bullet hits and destroys a sprite, an Explosion is added to the panel at the
	 * point where the bullet died.
	 */
	public void setExploding(boolean b) {
		exploding = true;
	}
	
	public boolean getExploding() {
		return exploding;
	}
	
	/**
	 * Kill the bullet and, if it is exploding, add an Explosion to the SpritePanel.
	 * This is ordinarily only called by a SpritePanel when the bullet hits another
	 * sprite.  (You could, however, call it for other reasons.)
	 */
	public void detonate() {
		die();
		if (exploding && getHome() != null) {
			getHome().addSprite( new Explosion(getX(), getY() ));
		}
	}

}
