package sprite;
import java.awt.*;

/**
 * A sprite that represents an explosion.  It has a limited lifetime and a maximum
 * size.  Over the course of its lifetime, it grows from 1/3 of the maximum size to
 * the maximum size.  Once its lifetime is over, it dies automatically and is removed
 * from the SpritePanel.  The lifetime and final size are determined when the Explosion
 * is constructed and cannot be changed.
 *
 */
public class Explosion extends Sprite {
	
	private int lifeTime;
	private double finalSize;
	private int age;  // for how many frames has this sprite existed so far?
	
	/**
	 * Create an Explosion sprite with lifetime equal to 10 frames and
	 * maximum size equal to 30.  The location of the explosion is (x,y).
	 */
	public Explosion( double x, double y ) {
		setImmuneToBullets(true);
		setLocation(x,y);
		age = 0;
		lifeTime = 10;
		finalSize = 30;
	}
	
	/**
	 * Create an Explosion sprite with specified position, lifetime, and final
	 * size.
	 */
	public Explosion( double x, double y, int lifeInFrames, int maxSize ) {
		setImmuneToBullets(true);
		setLocation(x,y);
		age = 0;
		lifeTime = lifeInFrames;
		finalSize = maxSize;
	}
	
	/**
	 * Note: This method is called by the SpritePanel that contains the
	 * explosion to get the sprite ready for the next frame, and is not meant 
	 * to be called for other reasons; it kills the sprite when the age of 
	 * the sprite exceeds its lifetime.
	 */
	public void prepareForNextFrame(int frameNumber) {
		age++;
		if (age > lifeTime)
			die();
	}
	
	public void draw(Graphics g) {
		double size = finalSize * ((double)age / lifeTime);
		g.setColor(Color.YELLOW);
		fillStar(g,10,getX(),getY(),size/2,size,13);
		g.setColor( new Color(255,120,0) );
		fillStar(g,12,getX(),getY(),size/3,2*size/3,5);
		g.setColor( Color.RED );
		fillStar(g,7,getX(),getY(),size/5,size/2,72);
		g.setColor(Color.YELLOW);
		fillStar(g,8,getX(),getY(),size/10,size/3,134);
	}

	private void fillStar(Graphics g, int points, double centerX, double centerY, 
			double innerRadius, double outerRadius, double startAngle) {
		double angle = startAngle - 90;
		Polygon poly = new Polygon();
		for (int i = 0; i < 2*points; i++) {
			double x,y;
			if (i % 2 == 0) {
				x = centerX + outerRadius * Math.cos(angle/180*Math.PI);
				y = centerY + outerRadius * Math.sin(angle/180*Math.PI);
			}
			else {
				x = centerX + innerRadius * Math.cos(angle/180*Math.PI);
				y = centerY + innerRadius * Math.sin(angle/180*Math.PI);
			}
			poly.addPoint( (int)x, (int)y );
			angle += 360.0/(2*points);
		}
		g.fillPolygon(poly);
	}

}
