package sprite;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.ArrayList;

/**
 *  A SpritePanel is a panel that can hold, display, and manage Sprites.  This class works
 *  closely with the Sprite class and with the Bullet class to implement basic Sprite behavior.
 *  A SpritePanel can run an animation, but the animation does not start automatically.
 *  After the SpritePanel has been created, it is necessary to call its go() method to start
 *  the animation.  (Animation is not necessary if the panel doesn't contain any sprites that
 *  move by themselves, that is, sprites that have non-zero velocity.  In addition, bullets will
 *  only kill sprites during an animation.)
 */
public class SpritePanel extends JPanel {

	private ArrayList<Sprite> sprites;  // Sprites that live in this SpritPanel, not including bullets.
	private ArrayList<Bullet> bullets;  // Bullet sprites that live in this panel.
	private Timer timer;                // Timer for driving the animation.
	private int frameNumber = 0;        // Current frame number, goes up by one in each frame.
	
	/**
	 * Creates the panel and the timer that will drive the animation.
	 * Sets the preferred size of the panel to be 800-by-600.
	 */
	public SpritePanel() {
		sprites = new ArrayList<Sprite>();
		bullets = new ArrayList<Bullet>();
		setPreferredSize(new Dimension(800,600));
		timer = new Timer(33, new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				doBasicUpdateForFrame();
				prepareForNextFrame(frameNumber);
				repaint();
			}
		});
	}
	
	/**
	 * This method is called once for each frame in an animation to give the
	 * panel a chance to update its state for the new frame.  It is not meant 
	 * to be called directly. Subclasses can override this method if needed.
	 * Note that the basic updating -- moving the sprites and checking if they
	 * are hit by bullets or have hit the edge of the panel -- is done elsewhere,
	 * not in this method.
	 */
	protected void prepareForNextFrame(int frameNumber) {
	}
	
	/**
	 * This method is called when the user presses a key on the keyboard, but
	 * only if that key has been set up by calling by calling this panel's
	 * installKeyPress() method.  This method is not meant to be called directly.
	 * It should be redefined in any subclass that does keyboard interaction.
	 * It specified what happens in response to the user pressing the key.
	 * @param keyStrokeString A string describing the key. This will be
	 * the same string that was "installed" by calling installKeyPress().
	 */
	protected void doKeyPress(String keyStrokeString) {
	}
	
	/**
	 * A subclass that wants to respond to the keyboard should override this
	 * method.  Key presses are only "seen" by the SpritePanel if the key is "installed"
	 * in the panel by calling this method.
	 * @param keyStrokeString A string describing the key.  Only certain
	 * values are allowed, including: "SPACE", for the space bar;
	 * "UP", "DOWN", "LEFT", and "RIGHT" for the arrow keys; "A", "B", ...
	 * "Z", for the letter keys; "0", "1", ..."9", for the number keys;
	 * "PERIOD", "COMMA", "TAB", "ENTER".  Note that keys will auto-repeat
	 * when they are held down by the user.  The method is called for
	 * each autorepeat as well as for the original keypress.
	 */
	protected void installKeyPress(String keyStrokeString) {
		final String cmd = keyStrokeString;
		getInputMap(WHEN_IN_FOCUSED_WINDOW).put(KeyStroke.getKeyStroke(cmd), cmd);
		getActionMap().put(cmd, new AbstractAction() { 
			public void actionPerformed(ActionEvent evt) {
				purge();
				doKeyPress(cmd);
				repaint();
			}
		});
	}
	
	/**
	 * This method is called in the paintComponent() method after the panel has
	 * been filled with the panel's background color but before any of the sprites
	 * are drawn.  The method in this class does nothing.  It can be overridden
	 * in a subclass that want's to have something in the picture behind the sprites.
	 * @param frameNumber the frame number for the frame that is being drawn.  The frameNumber
	 * only changes if an animation is in progress, and this method can be called 
	 * more than once with the same frame number.
	 */
	protected void drawBackground(Graphics g, int frameNumber) {
	}
	
	/**
	 * Removes all sprites, including bullets, from this SpritePanel.
	 */
	public void clear() {
		bullets.clear();
		sprites.clear();
	}

	/**
	 * Adds a sprite to this panel.  Sprites that have been added will be drawn
	 * in the panel.  If they have non-zero velocity, they will move automatically,
	 * as long as an animation is running.  The panel keeps two lists of sprites,
	 * one for bullets and one for other kinds of sprite, and it checks whether
	 * the bullets hits the sprites.  When a sprite dies for any reason, it is 
	 * automatically removed from the panel.
	 */
	public void addSprite(Sprite s) {
		if (s == null)
			throw new IllegalArgumentException("Can't have a null sprite.");
		if (s instanceof Bullet)
			bullets.add((Bullet)s);
		else
			sprites.add(s);
		s.setHome(this);
	}
	
	/**
	 * Tells how many non-bullet sprites are in this SpritePanel.  This can sometimes include "dead"
	 * sprites, since sprites are not removed as soon as they die.
	 */
	public int getSpriteCount() {
		return sprites.size(); 
	}
	
	/**
	 * Gets the i-th non-bullet sprite.  Throws an exception if i is not
	 * in the range 0 to getSpriteCount()-1.  It is possible for this method
	 * to return a "dead" sprite, since sprites are not removed as soon as 
	 * they die.
	 */
	public Sprite getSprite(int i) {
		return sprites.get(i);
	}
	
	/**
	 * Tells how many Bullet sprites are in this SpritePanel.
	 * @return
	 */
	public int getBulletCount() {
		return bullets.size(); 
	}
	
	/**
	 * Gets the i-th Bullet sprite.  Throws an exception if i is not
	 * in the range 0 to getBulletCount()-1.
	 */
	public Sprite getBullet(int i) {
		return bullets.get(i);
	}
	
	/**
	 * Sets the number of milliseconds per frame.  Only has an effect when
	 * an animation is running.  The default value is 33, for about 30
	 * frames per second.
	 * @param milliseconds time between frames.  The minimum value is 5.
	 * If a smaller value is specified, it is silently change to 5.
	 */
	public void setFrameDelay(int milliseconds) {
		if (milliseconds < 5)
			milliseconds = 5;
		timer.setDelay(milliseconds);
	}
	
	/**
	 * Returns the number of milliseconds between frames.
	 */
	public int getFrameDelay() {
		return timer.getDelay();
	}
	
	/**
	 * If an animation is running, this stops it.  It can be restarted by calling
	 * the go() method.  Has no effect if the animation is already paused.
	 */
	public void pause() {
		timer.stop();
	}

	/**
	 * Starts or restarts the animation.  If the animation is already running calling
	 * this method has no effect.
	 */
	public void go() {
		timer.start();
	}
	
	/**
	 * Tells whether an animation is running.
	 * @return true if an animation is running, false if not
	 */
	public boolean isPaused() {
		return !timer.isRunning();
	}
	
	protected void paintComponent(Graphics g) { 
		super.paintComponent(g);
		((Graphics2D)g).setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		drawBackground(g, frameNumber);
		for (int i = 0; i < sprites.size(); i++) {
			Sprite sprite = sprites.get(i);
			g.setColor(Color.BLACK);
			sprite.draw(g);
		}
		for (int i = 0; i < bullets.size(); i++) {
			Bullet bullet = bullets.get(i);
			bullet.draw(g);
		}
	}
	
	private void doBasicUpdateForFrame() { // handles basic per-frame processing
		frameNumber++;
		for (int i = 0; i < sprites.size(); i++) { // update position of sprites
			Sprite sprite = sprites.get(i);
			sprite.moveToNextFramePosition();
		}
		for (int i = 0; i < bullets.size(); i++) { // check for collisions with bullets
			Bullet bullet = bullets.get(i);
			bullet.moveToNextFramePosition();
			double bx = bullet.getX();
			double by = bullet.getY();
			if ( ! bullet.isDead() ) {
				for (int j = sprites.size() - 1; j >= 0; j--) {
					Sprite sprite = sprites.get(j);
					if ( !sprite.isDead() && sprite.contains(bx,by) && !sprite.isImmuneToBullets() ) {
						sprite.die();
						bullet.detonate();
						break;
					}
				}
			}
		}
		purge();
		for (int i = sprites.size()-1; i >= 0; i--) { // give sprites a chance to prepare for the next frame
			sprites.get(i).prepareForNextFrame(frameNumber);
		}
		for (int i = bullets.size()-1; i >= 0; i--) {  // give bullets a chance to prepare for the next frame
			bullets.get(i).prepareForNextFrame(frameNumber);
		}
	}
	
	private void purge() { // remove dead sprites
		for (int i = sprites.size()-1; i >= 0; i--) {  // discard any sprites that have died
			if (sprites.get(i).isDead())
				sprites.remove(i);
		}
		for (int i = bullets.size()-1; i >= 0; i--) {  // discard any bullets that have died
			if (bullets.get(i).isDead())
				bullets.remove(i);
		}
	}
		
}
